//----------------------------------------------------------------------------------------
// UDocumentPolevault.cp
// Copyright 1991-1992 Apple Computer, Inc.
//----------------------------------------------------------------------------------------

#ifndef __UDOCUMENTPolevault__
#include "UDocumentPolevault.h" 
#endif

#ifndef __UFILEBASEDDOCUMENT__
#include <UFileBasedDocument.h>
#endif

#ifndef __UCOMMANDPolevault__
#include "UCommandPolevault.h"
#endif

#ifndef __UPOLEVAULTVIEW__
#include "UPolevaultView.h"
#endif

#ifndef __UREMOTECOMMANDS__
#include "URemoteCommands.h"
#endif

#ifndef __AUXAESUITE__
#include "AUXAESuite.h"
#endif

#ifndef __UPolevaultDIALOGS__
#include "UPolevaultDialogs.h"
#endif

#ifndef __UPolevaultAPPLEEVENT__
#include "UPolevaultAppleEvent.h" 
#endif

#ifndef __UPRINTING__
#include <UPrinting.h>
#endif

#ifndef TOOLUTILS
#include "ToolUtils.h"
#endif

#ifndef PACKAGES
#include <Packages.h>
#endif


#ifndef __STRINGS__
#include <Strings.h>
#endif

#ifndef __PLSTRINGFUNCS__
#include <PLStringFuncs.h>
#endif

//constants
TDocumentPolevault* gDocList[16];
int nDocs = 0;

const short kDefaultSize = 40;
const OSType kDefaultName = 'newV';

extern "C" void debugf(char*, ...);
//========================================================================================
// CLASS TDocumentPolevault
//========================================================================================
/****************************** TDocumentMandel ******************************/

pascal OSErr MandelEventHandler(AppleEvent msg, AppleEvent /* reply */, long /* userData */)
{
	DescType 	realType;
	Size 		realSize;
	OSErr		osErr;
	int			id;
	
	osErr = AEGetParamPtr(msg, 'doc#', typeInteger, realType, (Ptr) &id, sizeof(id), realSize);

	if (osErr == noErr)
		if (gDocList[id])
			gDocList[id]->ProcessEvent(msg);

	return noErr;
}

typedef struct TCPAddress 
{
	char 		name[32];			// e.g. "apple.com" or "90.134.12.15"
	char		app[32];			// e.g. "fooserver"
	long		addr;				// converted from "name" by name translator
}
TCPAddress, *TCPAddrPtr;


//----------------------------------------------------------------------------------------
// TDocumentPolevault::Initialize: 
//----------------------------------------------------------------------------------------
#pragma segment AOpen
pascal void TDocumentPolevault::Initialize() // Override 
{
	inherited::Initialize();
	
	fPolevaultView 		= NULL;
	fWindowTitle		= gEmptyString;
	fHeartBeatCounter	= TickCount()/60;    //gotta do it here because DoIdle waits for nobody!
	fPortRefNum			= NULL;
	int 			i;
	
//mandel doc stuff	

	gDocList[nDocs] = this;
	myDocID = nDocs++;
	fMaxQueueSize = MaxQueueSize;
	
/*	nRows = 10;
	nCols = 10;
	
	rowPix = 32;
	colPix = 32;
*/
	nRows = 8;
	nCols = 8;
	
	rowPix = 64;
	colPix = 64;

	queued = 0;
	nAddrs = 0;
	lastAddr = 0;

	fCTable = GetCTable(128);
	
	fOffGrafPtr = new CGrafPort;
	(*(fOffGrafPtr->portPixMap))->baseAddr = nil;
	
	transaction = 1;
		
	rMin = -2.0;
	rMax = 0.5;
	iMin = -1.25;
	iMax = 1.25;

//	SetBounds();
	
	for (i = 0; i < kMaxHosts; i++)
		queueCount[i] = 0;

	
}

//----------------------------------------------------------------------------------------
// TDocumentPolevault::IDocumentPolevault: 
//----------------------------------------------------------------------------------------
#pragma segment AOpen
pascal void TDocumentPolevault::IDocumentPolevault(TFile* itsFile,
												OSType itsCreator)
{

	this->IFileBasedDocument(itsFile,itsCreator);
} 

//----------------------------------------------------------------------------------------
// TDocumentPolevault::Close: 
//----------------------------------------------------------------------------------------
pascal void TDocumentPolevault::Disconnect()
{
//	this->fAEAddressDesc.descriptorType = typeNull;
//	this->fAEAddressDesc.dataHandle = NULL;
	nAddrs = 0;

} // TDocumentPolevault::Close 

//----------------------------------------------------------------------------------------
// TDocumentPolevault::Close: 
//----------------------------------------------------------------------------------------
pascal void TDocumentPolevault::Close()
{
	CWindowIterator iter(this);

	for (TWindow* aWindow = iter.FirstWindow(); iter.More(); aWindow = iter.NextWindow())
		aWindow->CloseAndFree();
	
	if (Connected())
		{
		for (int i = 0; i < nAddrs; i++)
		{
			TQuitCmd* aQuitCmd = new TQuitCmd;
			aQuitCmd->IQuitCmd(this, i);
			aQuitCmd->DoIt();    //otherwise the doc is gone if I post it and kaboom!
		}
		this->Disconnect();
		}
} // TDocumentPolevault::Close 

//----------------------------------------------------------------------------------------
// TDocumentPolevault::Free: 
//----------------------------------------------------------------------------------------
#pragma segment AClose
pascal void TDocumentPolevault::Free() // Override 
{
	Ptr bits = (*(fOffGrafPtr->portPixMap))->baseAddr;
	CloseCPort(fOffGrafPtr);
	delete fOffGrafPtr;
	DisposePtr(bits);
	
	for (int i = 0; i < nAddrs; i++)
	{
		AEDisposeDesc(hostAddr[i]);
	}
	
	AERemoveEventHandler('A/UX', 'mndl', (ProcPtr) MandelEventHandler, false);
	AERemoveEventHandler(kCoreEventClass, kAEAnswer, (ProcPtr) MandelEventHandler, false);
		
	inherited::Free();
}

//----------------------------------------------------------------------------------------
// TDocumentPolevault::DoMakeViews: 
//----------------------------------------------------------------------------------------
#pragma segment AOpen
pascal void TDocumentPolevault::DoMakeViews(Boolean /*forPrinting*/) // Override 
{
	OSErr		osErr;
	TWindow* aWindow = NULL;
	TStdPrintHandler* aHandler = NULL;
	TPolevaultView* aPolevaultView = NULL;
//	CPoint resize;
//	resize.v = 20;
//	resize.h = 496;

	FailNIL(aWindow = gViewServer->NewTemplateWindow(kDefaultWindowID, this));
//	aWindow->Zoom(inZoomOut);
//	aWindow->SetResizeLimits(resize,resize);

	aPolevaultView = (TPolevaultView*) (aWindow->FindSubView('SKEL'));	// Must cast because FindSubView returns TView
	fPolevaultView = aPolevaultView;

	fPolevaultView->SetFrame(VRect(0, 0, nCols*colPix, nRows*rowPix), false);
	aWindow->SetFrame(VRect(10, 10, 16 + 10 + nCols*colPix, 16 + 10 + nRows*rowPix), true);

	this->DoMenuCommand(cGetPPCServerCmd);      // launches ppc browser on startup  (actually on new document)
	SetupPort();
	SetBounds();


	if (!this->Connected())
		Failure(0,0);

	
	aHandler = new TStdPrintHandler;
	aHandler->IStdPrintHandler(this,			// its document 
							   aPolevaultView,		// its view 
							   !kSquareDots,	// does not have square dots 
							   kFixedSize,		// horzontal page size is fixed 
							   kFixedSize);		// vertical page size is fixed 

	fHeartBeatCounter	= TickCount()/60;
	osErr = AEInstallEventHandler('A/UX', 'mndl', (ProcPtr) MandelEventHandler, 0, false);
	osErr = AEInstallEventHandler(kCoreEventClass, kAEAnswer, (ProcPtr) MandelEventHandler, 0, false);
} // TDocumentPolevault::DoMakeViews 

//----------------------------------------------------------------------------------------
// TDocumentPolevault::DoMenuCommand: This method is overridden to handle menu items which
// are enabled when this document is in the target chain. In this example, this is true
// when the document is open and its window is the active window. The inherited method
// should always be called so that MacApp can allow successor objects in the target chain
// (i.e. the application) to handle THEIR menu items.
//----------------------------------------------------------------------------------------
#pragma segment ASelCommand
pascal void TDocumentPolevault::DoMenuCommand(CommandNumber aCommandNumber) // Override 
{

	switch (aCommandNumber) 
	{
		case cGetPPCServerCmd: 
			TLookupLauncher* aCommand = new TLookupLauncher;
			aCommand->ILookupLauncher(aCommandNumber,this);
			aCommand->DoIt();
			//this->PostCommand(aCommand);
			this->WakeupServer();   //wakes them all up!

			break;
		case cSetLook1:
			fMaxQueueSize = 1;
			break;
			
		case cSetLook2:
			fMaxQueueSize = 2;
			break;
			
		case cSetLook4:
			fMaxQueueSize = 4;
			break;
			
		case cSetLook8:
			fMaxQueueSize = 8;
			break;
			
		case cSetLook16:
			fMaxQueueSize = 16;
			break;
			
		case cSetPix8:
	currRow = 0;
	currCol = 0;
			nRows = nRows * rowPix / 8;
			nCols = nCols * colPix / 8;

			rowPix = 8;
			colPix = 8;
			Zoom(VPoint(0,0,),VPoint(nCols*colPix, nRows*rowPix));
			break;

		case cSetPix16:
	currRow = 0;
	currCol = 0;
			nRows = nRows * rowPix / 16;
			nCols = nCols * colPix / 16;

			rowPix = 16;
			colPix = 16;
			Zoom(VPoint(0,0,),VPoint(nCols*colPix, nRows*rowPix));
			break;

		case cSetPix32:
	currRow = 0;
	currCol = 0;
			nRows = nRows * rowPix / 32;
			nCols = nCols * colPix / 32;

			rowPix = 32;
			colPix = 32;
			Zoom(VPoint(0,0,),VPoint(nCols*colPix, nRows*rowPix));
			break;

		case cSetPix64:
	currRow = 0;
	currCol = 0;
			nRows = nRows * rowPix / 64;
			nCols = nCols * colPix / 64;

			rowPix = 64;
			colPix = 64;
			Zoom(VPoint(0,0,),VPoint(nCols*colPix, nRows*rowPix));
			break;

		case cSetGrid320:
			currRow = 0;
			currCol = 0;
			nRows = 320 / rowPix;
			nCols = 320 / colPix;

			SetBounds();
			CalcWindowSize();
			break;
		case cSetGrid512:
			currRow = 0;
			currCol = 0;
			nRows = 512 / rowPix;
			nCols = 512 / colPix;

			SetBounds();
			CalcWindowSize();
			break;

		case cSetGrid640:
			currRow = 0;
			currCol = 0;
			nRows = 640 / rowPix;
			nCols = 640 / colPix;
			SetBounds();
			CalcWindowSize();
			break;
			
		default:
			inherited::DoMenuCommand(aCommandNumber);
			break;
	}
} 


//----------------------------------------------------------------------------------------
// TDocumentPolevault::\upMenus: This method is overridden to enable menu items which
// should be enabled when this object is in the target chain. MacApp initially disables
// all menu items, then lets the objects in the target chain enable those items they
// handle.
//
// A document object is in the target chain when its window is the active window.
//
// The inherited method is called so that TDocument can enable document-level menu items
// like "Save ". This also ensures that objects further up the target chain (the
// application) can set up THEIR menus.
//----------------------------------------------------------------------------------------
#pragma segment ARes
//#define cSetLookMenu					404
//#define mSetLookMenu					6
pascal void TDocumentPolevault::DoSetupMenus() // Override 
{
	inherited::DoSetupMenus();
	
	if (this->Connected())
		{
		EnableCheck(cSetLook1, true, fMaxQueueSize == 1);
		EnableCheck(cSetLook2, true, fMaxQueueSize == 2);
		EnableCheck(cSetLook4, true, fMaxQueueSize == 4);
		EnableCheck(cSetLook8, true, fMaxQueueSize == 8);
		EnableCheck(cSetLook16, true, fMaxQueueSize == 16);
		EnableCheck(cSetPix8, true, rowPix == 8);
		EnableCheck(cSetPix16, true, rowPix == 16);
		EnableCheck(cSetPix32, true, rowPix == 32);
		EnableCheck(cSetPix64, true, rowPix == 64);
		EnableCheck(cSetGrid320, true, rowPix*nRows == 320);
		EnableCheck(cSetGrid512, true, rowPix*nRows == 512);
		EnableCheck(cSetGrid640, true, rowPix*nRows == 640);
			 	
		}
} 
//----------------------------------------------------------------------------------------
#pragma segment ARes
pascal void TDocumentPolevault::DoPostMakeViews(Boolean forPrinting)
{
	if (Connected())
	{
		this->SetTitle(this->fWindowTitle);
		inherited::DoPostMakeViews(forPrinting);
	
	}
	else
		this->Close();
		
fHeartBeatCounter	= TickCount()/60;
} // TDocument::DoPostMakeViews 

//----------------------------------------------------------------------------------------
// TDocumentPolevault::Connected: 
//----------------------------------------------------------------------------------------

#pragma segment ARes
pascal Boolean TDocumentPolevault::Connected() 
{
	return (nAddrs > 0);
}
//----------------------------------------------------------------------------------------
// TDocumentPolevault::HaveAHeartBeat: 
//----------------------------------------------------------------------------------------

#pragma segment ARes
pascal void TDocumentPolevault::HaveAHeartBeat() 
{
	fHeartBeatCounter = TickCount()/60;
}
//----------------------------------------------------------------------------------------
// TDocumentPolevault::WakeupServer: 
//----------------------------------------------------------------------------------------
#pragma segment ARes
pascal void TDocumentPolevault::WakeupServer()
{
		for (int i = 0; i < this->nAddrs; i++)
		{
			TWakeupCommand* aCommand = new TWakeupCommand;
			aCommand->IWakeupCommand(this, i);
			this->PostCommand(aCommand);
		}
}
//----------------------------------------------------------------------------------------
// TDocumentPolevault::CheckPulse: 
//----------------------------------------------------------------------------------------

#pragma segment ARes
pascal Boolean TDocumentPolevault::CheckPulse() 
{
	long now = TickCount()/60;
	
	if (((now - this->fHeartBeatCounter) > 120) && (!this->fWindowTitle.IsEmpty()) )      //2 minutes
	{

		CStr255 theOtherString	;
		
		theOtherString = this->fWindowTitle;
		this->fWindowTitle	= gEmptyString;      //this is my flag to let me know that I have one posed already
		
		PolevaultErrorBoxNumber(kCheckPulseError, theOtherString);
		
		return FALSE;   //this doc is a gonner 
	}
	else if ((now - this->fHeartBeatCounter) > 60  &&  (!this->fWindowTitle.IsEmpty()))      //1 minute
		{
		SysBeep(2);
		return TRUE;    //this doc is still ok
		}
	return TRUE;        //steady as she goes!
}

//----------------------------------------------------------------------------------------
// TDocumentPolevault::SetPortRefNum: 
//----------------------------------------------------------------------------------------
#pragma segment ADoCommand
pascal void TDocumentPolevault::SetPortRefNum(short thePortRefNum) 
{
	fPortRefNum = thePortRefNum;
}
//----------------------------------------------------------------------------------------
// TDocumentPolevault::GetPortRefNum: 
//----------------------------------------------------------------------------------------
#pragma segment ADoCommand
pascal short TDocumentPolevault::GetPortRefNum() 
{
	return fPortRefNum;
}

//----------------------------------------------------------------------------------------
#pragma segment ADoCommand
pascal void TDocumentPolevault::SetTitle(const CStr255& aTitle)
{
	inherited::SetTitle(aTitle);
	((TWindow*)fPolevaultView->GetWindow())->SetTitle(aTitle);
}
//----------------------------------------------------------------------------------------
#pragma segment ADoCommand
pascal void TDocumentPolevault::SetfWindowTitle(const CStr255& aTitle)
{
	this->fWindowTitle = aTitle;
}

//----------------------------------------------------------------------------------------
#pragma segment AVGRes
pascal Boolean TDocumentPolevault::LocateDocument(long theSessionID)
{
	return fPortRefNum == theSessionID;
}





void TDocumentPolevault::RemoteRequest()
{
	AppleEvent 	theAE;
	AppleEvent 	theReply;
	OSErr 		osErr;
	int			i;
	
	double r1 = rMin + currCol * dr;
	double r2 = r1 + dr;
	double i1 = iMin + currRow * di;
	double i2 = i1 + di;
	
	// ship reference to self, along with real params, to server
	
	// debugf("RemoteRequest of (%d, %d), queued=%d;g", currRow, currCol, queued);
	
	do 
	{
		lastAddr = (lastAddr + 1) % nAddrs;
	}
	while (queueCount[lastAddr] == fMaxQueueSize);
	
	queued++;
	queueCount[lastAddr]++;
	
	osErr = AECreateAppleEvent(kAEAUXSuite, kAEmndlMsg, hostAddr[lastAddr], 
								kAutoGenerateReturnID, kAnyTransactionID, theAE);
		
	////	stuff AppleEvent 
	
	AEPutParamPtr(theAE, 'trx#', typeInteger, (Ptr) & transaction, sizeof(int));
	AEPutParamPtr(theAE, 'doc#', typeInteger, (Ptr) & myDocID, sizeof(int));
	AEPutParamPtr(theAE, 'srv#', typeInteger, (Ptr) & lastAddr, sizeof(int));
	
	AEPutParamPtr(theAE, 'row#', typeInteger, (Ptr) & currRow, sizeof(int));
	AEPutParamPtr(theAE, 'col#', typeInteger, (Ptr) & currCol, sizeof(int));
	
	AEPutParamPtr(theAE, 'rows', typeInteger, (Ptr) & rowPix, sizeof(int));
	AEPutParamPtr(theAE, 'cols', typeInteger, (Ptr) & colPix, sizeof(int));
	
	AEPutParamPtr(theAE, 'rmin', typeFloat, (Ptr) & r1, sizeof(double));
	AEPutParamPtr(theAE, 'rmax', typeFloat, (Ptr) & r2, sizeof(double));
	AEPutParamPtr(theAE, 'imin', typeFloat, (Ptr) & i1, sizeof(double));
	AEPutParamPtr(theAE, 'imax', typeFloat, (Ptr) & i2, sizeof(double));

	// debugf("Shipping request for [%d, %d];g", currRow, currCol);
		
	////	send AppleEvent
	osErr = AESend(theAE, theReply, kAEQueueReply, kAENormalPriority, 120, nil, nil);
	if (osErr != noErr)
	{
//		debugf("AESend: error %d;g", osErr);
		
		////	take this address out, mark job as not getting done
		
		currCol--;							// 	meaning: we did not process this square	
		AEDisposeDesc(hostAddr[lastAddr]);	//	address-be-gone
		nAddrs--;							// 	one less address
		
		for (i = lastAddr; i < nAddrs; i++)
		{
			queueCount[i] = queueCount[i+1];
			hostAddr[i] = hostAddr[i+1];
		}
	}
	else
		GrayOut(currRow, currCol);
		
	////	deallocate the event
	
	AEDisposeDesc(theAE);
}

void TDocumentPolevault::ProcessEvent(AppleEvent msg)
{
	DescType 	realType;
	Size 		realSize;
	OSErr		osErr;
	// int			id;
	char*	 	data;
	int 		r, c;
	int			i, trid;
	long		bufSize;
	
	////	mark this guy as having a smaller queue
	
	bufSize = rowPix * colPix;
	data = NewPtr( bufSize );
	
	osErr = AEGetParamPtr(msg, 'srv#', typeInteger, realType, (Ptr) & i, sizeof(int), realSize);
	
	queueCount[i]--;
	queued--;
	
	////	retrieve 'data' value
	
	osErr = AEGetParamPtr(msg, 'data', typeChar, realType, (Ptr) data, bufSize, realSize);
	osErr = AEGetParamPtr(msg, 'row#', typeInteger, realType, (Ptr) &r, sizeof(r), realSize);
	osErr = AEGetParamPtr(msg, 'col#', typeInteger, realType, (Ptr) &c, sizeof(c), realSize);
	osErr = AEGetParamPtr(msg, 'trx#', typeInteger, realType, (Ptr) &trid, sizeof(trid), realSize);

	if (trid != transaction)
		return;
	
	FillSquare(r, c, data);
	
	DisposePtr( data );
}


void TDocumentPolevault::FillSquare(int r, int c, char* data)
{
	PixMap 	myMap;
	CRect 	srcCRect(0, 0, colPix, rowPix);
	CPoint 	offset(c * colPix, r * rowPix);
	CRect 	dstCRect = srcCRect + offset;

	myMap.baseAddr = (Ptr) data;
	myMap.rowBytes = 0x8000 + rowPix;
	myMap.bounds = srcCRect;
	myMap.pmVersion = 0;
	myMap.packType = 0;
	myMap.packSize = 0;
	myMap.hRes = myMap.vRes = (72 << 8);
	myMap.pixelType = 0;
	myMap.pixelSize = 8;
	myMap.cmpCount = 1;
	myMap.cmpSize = 8;
	myMap.planeBytes = 0;
	myMap.pmReserved = 0;
	
	GrafPtr port;	
	GDHandle device;
	
	GetPort(port);
	device = GetGDevice();
	
	HLock((Handle) fOffGrafPtr->portPixMap);

	SetPort((GrafPtr) fOffGrafPtr);
	SetGDevice(fOffGrafDev);
	
	myMap.baseAddr = (Ptr) data;
	myMap.pmTable = (*(fOffGrafPtr->portPixMap))->pmTable = fCTable;

	CopyBits((BitMap&) myMap, (BitMap&)(**(fOffGrafPtr->portPixMap)), srcCRect, dstCRect, srcCopy, nil);
		
	SetPort(port);
	SetGDevice(device);
	
	HUnlock((Handle) fOffGrafPtr->portPixMap);

	TView* view;
	TWindow* wind;
	CWindowIterator List(this);
	
	for (wind = List.FirstWindow(); List.More(); wind = List.NextWindow())
	{
		if (view = wind->FindSubView('SKEL'))
			view->InvalidateRect(dstCRect);
	}
}


void TDocumentPolevault::GrayOut(int r, int c)
{
	char* data = NewPtr((1+rowPix) * (1+colPix));	
	
	memset(data, 245 - lastAddr*rowPix, rowPix * colPix);
//	memset(data, 245 - lastAddr*32, rowPix * colPix);
	FillSquare(r, c, data);

	DisposePtr(data);
}


pascal Boolean TDocumentPolevault::DoIdle(IdlePhase)
{
	if (queued < nAddrs * fMaxQueueSize)
	{
		if (currRow < nRows)
		{
			if (currCol < nCols)
			{
				RemoteRequest();			
				currCol++;
			}
			else
			{
				currRow++;
				currCol = 0;
			}
		}
	}
	if (nAddrs > 0)
	{
		return false; // did not free myself
	}
	else
	{
//		Free();
		Close();
		return true;
	}
}


void TDocumentPolevault::Zoom(VPoint topleft, VPoint botright)
{
	double rPixels = (double) rowPix * nRows;
	double cPixels = (double) colPix * nCols;
	
	double rMinOld = rMin; 
	double iMinOld = iMin;
	double rMaxOld = rMax;
	double iMaxOld = iMax;
	
	rMin = rMinOld + (rMaxOld - rMinOld) * topleft.h / cPixels;
	iMin = iMinOld + (iMaxOld - iMinOld) * topleft.v / rPixels;

	rMax = rMinOld + (rMaxOld - rMinOld) * botright.h / cPixels;
	iMax = iMinOld + (iMaxOld - iMinOld) * botright.v / rPixels;

	dr = (rMax - rMin) / nCols;
	di = (iMax - iMin) / nRows;

	currRow = 0;
	currCol = 0;
	transaction++;
	
	CRect dstCRect = (*(fOffGrafPtr->portPixMap))->bounds;

	// black out the offscreen bitmap
	
	GrafPtr port;	
	GetPort(port);
	
	SetPort((GrafPtr) fOffGrafPtr);
	
	FillRect(dstCRect, &qd.dkGray);
	
	SetPort(port);
	
	// refresh the visible rects
	
	TView* view;
	TWindow* wind;
	CWindowIterator List(this);
	
	for (wind = List.FirstWindow(); List.More(); wind = List.NextWindow())
	{
		if (view = wind->FindSubView('SKEL'))
			view->InvalidateRect(dstCRect);
	}	

}

#define kOkay 1
#define kCancel 2
#define kPixInfo 3
#define kGridInfo 4
extern "C" int atoi(char*);

void TDocumentPolevault::SetBounds()
{

	currRow = 0;
	currCol = 0;
	
	dr = (rMax - rMin) / nCols;
	di = (iMax - iMin) / nRows;

	// OFFSCREEN BITMAP JUNK HERE (duplicated from above, icky, FIXME)
	
	// debugf("Bitmap stuff begins;g");
/*	
	fOffGrafPtr = new CGrafPort;
	
	CRect globalRect(-32000, -32000, 32000, 32000);
	CRect localRect(0, 0, nCols*colPix, nRows*rowPix);
	
	GDHandle oldDevice;
	oldDevice = GetGDevice();
	
	fOffGrafDev = GetMaxDevice(globalRect);
	SetGDevice(fOffGrafDev);
	OpenCPort(fOffGrafPtr);

*/	
	GDHandle oldDevice;
	oldDevice = GetGDevice();

	CRect localRect(0, 0, nCols*colPix, nRows*rowPix);
	int theDepth = (*(fOffGrafPtr->portPixMap))->pixelSize;
	
	// Determine size of bitmap data
	long offRowBytes = (((theDepth * (localRect.right)) + 15) >> 4) << 1;
	long sizeOfOff = (long) (localRect.bottom) * offRowBytes;

	if ((*(fOffGrafPtr->portPixMap))->baseAddr != nil)
		DisposePtr( (*(fOffGrafPtr->portPixMap))->baseAddr );   //this blows up on a blackbird>!!  caw cawboom!

	Ptr myBits = NewPtr(sizeOfOff);
	
	// Stuff into the port pix map
	
	(*(fOffGrafPtr->portPixMap))->baseAddr = myBits;
	(*(fOffGrafPtr->portPixMap))->rowBytes = (short) offRowBytes + 0x8000;
	(*(fOffGrafPtr->portPixMap))->bounds = localRect;
	
	// uninitialized state: dark gray
	GrafPtr port;	
	GetPort(port);
	SetPort((GrafPtr) fOffGrafPtr);
	FillRect(localRect, &qd.dkGray);
	SetPort(port);
	SetGDevice(oldDevice);

	// END OFFSCREEN BITMAP JUNK

}

void TDocumentPolevault::CalcWindowSize()
{
	
	TView* view;
	TWindow* wind;
	CWindowIterator List(this);
	
	for (wind = List.FirstWindow(); List.More(); wind = List.NextWindow())
	{
		wind->Resize(VPoint(16 + nCols*colPix, 16 + nRows*rowPix), true);
//		wind->SetFrame(VRect(32, 32, 16 + 32 + nCols*colPix, 16 + 32 + nRows*rowPix), true);

		if (view = wind->FindSubView('SKEL'))
			view->SetFrame(VRect(0, 0, nCols*colPix, nRows*rowPix), true);
	}	
	Zoom(VPoint(0,0,),VPoint(nCols*colPix, nRows*rowPix));
}

void TDocumentPolevault::SetupPort()
{
	fOffGrafPtr = new CGrafPort;
	
	CRect globalRect(-32000, -32000, 32000, 32000);
//	CRect localRect(0, 0, nCols*colPix, nRows*rowPix);
	
	GDHandle oldDevice;
	oldDevice = GetGDevice();
	
	fOffGrafDev = GetMaxDevice(globalRect);
	SetGDevice(fOffGrafDev);
	OpenCPort(fOffGrafPtr);

	SetGDevice(oldDevice);
}