/*
	UDocumentPolevault.h
	
	Copyright 1991-1992 Apple Computer, Inc.
 	All rights reserved

	Change History:
	
 */

#ifndef __UDOCUMENTPolevault__
#define __UDOCUMENTPolevault__

#ifndef __UFILEBASEDDOCUMENT__
#include <UFileBasedDocument.h>
#endif

class PascalObj TPolevaultView;
class PascalObj TApplicationPolevault;

extern char*	gPolevaultVersion = "00.00";
extern TApplicationPolevault* gApplicationPolevault;
//----------------------------------------------------------------------------------------
// Constants:
const ResNumber	kCmdAEEvents =			1001;		// 'aedt' resource constant
const ResNumber	kAlertUserToError =		1001;		// error string rsrc ID
const OSType	kAECmdSuiteID = 		'A/UX';		// AE class
const OSType 	kAEunixType = 			'unix';
const long 	BUFSIZ	= 				10000;

const short	kPolevaultWindowID = 		1002;     
//const short	kPolevaultWindowID = 		5500;     //ice pick  DiscusIcePick.rsrc
const short	kConfirmBox =			5996;
const short	kInfoBox =				5997;
const short	kErrorBox =				5998;
//const short	kStatusBox = 			5999;

extern pascal void PolevaultErrorBox(CStr255& theMessage, const CStr255& param1=gEmptyString, const CStr255& param2=gEmptyString, const CStr255& param3=gEmptyString);
extern pascal void PolevaultErrorBoxNumber(int theErrorNum, const CStr255& param1=gEmptyString, const CStr255& param2=gEmptyString, const CStr255& param3=gEmptyString);
extern pascal void PolevaultInfoBox(CStr255& theMessage, const CStr255& param1=gEmptyString, const CStr255& param2=gEmptyString, const CStr255& param3=gEmptyString);
extern pascal void PolevaultInfoBoxNumber(int theErrorNum, const CStr255& param1=gEmptyString, const CStr255& param2=gEmptyString, const CStr255& param3=gEmptyString);
extern pascal Boolean PolevaultConfirmBox(CStr255& theMessage, const CStr255& param1=gEmptyString, const CStr255& param2=gEmptyString, const CStr255& param3=gEmptyString);
extern pascal Boolean PolevaultConfirmBoxNumber(int theErrorNum, const CStr255& param1=gEmptyString, const CStr255& param2=gEmptyString, const CStr255& param3=gEmptyString);
extern pascal void SubstituteDialogText(CStr255& theMessage, const CStr255& param1=gEmptyString, const CStr255& param2=gEmptyString, const CStr255& param3=gEmptyString);

// Error constants for PolevaultErrorBox - or whatever
const			kUnknownPolevaultError =	3;     //also sent by server
const			kCheckPulseError =		8;
const			kBadVersionError =		9;
const			kNoVersionError = 		10;
 
// Commands
const CommandNumber cCommandHandledByDocument = 401;
const CommandNumber cSetLook1	= 600;
const CommandNumber cSetLook2	= 601;
const CommandNumber cSetLook4	= 602;
const CommandNumber cSetLook8	= 603;
const CommandNumber cSetLook16	= 604;

const CommandNumber cSetPix8	= 500;
const CommandNumber cSetPix16	= 501;
const CommandNumber cSetPix32	= 502;
const CommandNumber cSetPix64	= 503;

const CommandNumber cSetGrid320	= 700;
const CommandNumber cSetGrid512	= 701;
const CommandNumber cSetGrid640	= 702;

// Files
const OSType kSignature = 'MNDL';			// File-type code used for document files	(also in .r)

#define kMaxHosts	8
#define MaxQueueSize	2
//----------------------------------------------------------------------------------------
// TDocumentPolevault
//----------------------------------------------------------------------------------------

class TDocumentPolevault: public TFileBasedDocument
{
public:
	int nRows, nCols;
	int rowPix, colPix;
	int currRow, currCol;
	double rMin, rMax, iMin, iMax;
	double dr, di;
	int transaction;
	AEAddressDesc hostAddr[kMaxHosts];
	int queueCount[kMaxHosts];
	int nAddrs;
	int lastAddr;
	
	CGrafPtr fOffGrafPtr;
	GDHandle fOffGrafDev;
	CTabHandle fCTable;
	
	int fMaxQueueSize;
	int queued;
	int myDocID;

	TPolevaultView*			fPolevaultView;
	long					fHeartBeatCounter;
	short					fPortRefNum;
private:
	CStr255 				fWindowTitle;
	
public:		
	// Construction/Destruction
	virtual pascal void 		Initialize(); // Override
	
	virtual pascal void  	IDocumentPolevault(TFile* itsFile, OSType itsCreator);
//	virtual pascal void  	DoInitialState(); // Override
	virtual pascal Boolean 	DoIdle(IdlePhase);
	virtual pascal void  		Free(); // Override
	virtual pascal void  		Close(); // Override
	virtual pascal void  		Disconnect(); // Override
			
	// Views
	virtual pascal void  		DoMakeViews(Boolean forPrinting); // Override
	// Commands
	virtual pascal void  		DoSetupMenus(); // Override
	virtual pascal void  		DoMenuCommand(CommandNumber aCommandNumber); // Override
	virtual pascal void			DoPostMakeViews(Boolean forPrinting);
	
	//
	virtual pascal void 		SetPortRefNum(short thePortRefNum);
	virtual pascal short 		GetPortRefNum();
	virtual pascal Boolean 		Connected();
	virtual pascal void 		HaveAHeartBeat();
	virtual pascal Boolean 		CheckPulse();
	virtual pascal void 		WakeupServer();

	virtual pascal void 		SetTitle(const CStr255& aTitle);	
	virtual pascal void 		SetfWindowTitle(const CStr255& aTitle);	

	virtual pascal Boolean		LocateDocument(long theSessionID);	

	// Mandel-specific stuff
	
	void GrayOut(int r, int c);
	void SetData(char* data, int r, int c, int trid);
	void RemoteRequest();
	void Zoom(VPoint, VPoint);
	void ProcessEvent(AppleEvent msg);
//	void SetAddrs();     put this stuff into the lookuplauncher
	void SetBounds();
	void SetupPort();
	void CalcWindowSize();
	void FillSquare(int r, int c, char* data);

};

#endif
