// URemoteCommands.cp
// Copyright 1991-1992 Apple Computer, Inc.


#ifndef __UDOCUMENTPolevault__
#include "UDocumentPolevault.h"
#endif

#ifndef __AUXAESUITE__
#include "AUXAESuite.h"
#endif

#ifndef __UCOMMANDPolevault__
#include "UCommandPolevault.h"
#endif

#ifndef __UREMOTECOMMANDS__
#include "URemoteCommands.h"
#endif

#ifndef __PolevaultAPPLEEVENT__
#include "UPolevaultAppleEvent.h"
#endif

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef __STDIO__
#include <stdIO.h>
#endif

#ifndef __STDLIB__
#include <StdLib.h>
#endif

#ifndef __STRING__
#include <String.h>
#endif

//--------------------------------------------------------------------------------------------------
// superclasss for  client commands
//--------------------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------------------
#pragma segment ACommand
pascal TAppleEvent* TPolevaultClientCommand::SendMessage()
{
	return inherited::SendMessage();
}



//--------------------------------------------------------------------------------------------------
#pragma segment ACommand
pascal void TPolevaultClientCommand::IPolevaultClientCommand(TDocumentPolevault* itsDocument,
														const int theKey,
														TObject* toNotify,  int whichServer)
{
	this->IClientCommand(cRemoteCommand, itsDocument, kCantUndo, kCausesChange, toNotify);

	FailInfo fi;
	if (fi.Try())
	{
		TPolevaultAppleEvent* aMessage = new TPolevaultAppleEvent;		// create the message.
		
		aMessage->IPolevaultAppleEvent(kAECmdSuiteID, (AEEventID)theKey, itsDocument->hostAddr[whichServer], kAEWaitReply, itsDocument->GetPortRefNum() );
		fMessage = aMessage;
		fi.Success();
	}
	else
	{
		this->Free();
		fi.ReSignal();
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment ACommand
pascal void TPolevaultClientCommand::ProcessReply(TAppleEvent* theReply)
{
	char status[1];
	char string[1024];
	long actualSize;
	DescType actualCode;
	
	if (theReply == NULL)
		return;
		
	inherited::ProcessReply(theReply);
	
	AppleEvent theMessage = theReply->fMessage;

	OSErr err = AEGetParamPtr(theMessage, keyErrorNumber, typeChar, actualCode, (Ptr)status, sizeof(char), actualSize);

	if (err == -128   /*userCanceledErr*/)
		return;
	else 
	  if (err != noErr)
		{
		CStr255 theError;
		theError = "There was an error sending the last AppleEvent!";
		PolevaultErrorBox(theError);
		return;
		}

	if (*status == 'S') 
		{
		return;
		}
					
	theReply->ReadParameterPtr(keyErrorString, typeChar, actualCode,(Ptr)string, 1024, actualSize);
	string[actualSize] = NULL;

	// Wierdness for converting AE/UNIX strings to Mac strings.
	for (char* ptr = string; *ptr != NULL; ptr++)
		if (*ptr == '\r')
			*ptr = '\n';
	
	CStr255 tmpString = string;
	PolevaultErrorBox(tmpString);			

}

//--------------------------------------------------------------------------------------------------
// kAEQuit
//--------------------------------------------------------------------------------------------------
#pragma segment ACommand
pascal void TQuitCmd::IQuitCmd(TDocumentPolevault* itsDoc,  int whichServer)	
{
	this->IClientCommand(cRemoteCommand, itsDoc, kCantUndo, kCausesChange, NULL);

	FailInfo fi;
	if (fi.Try())
	{
		TPolevaultAppleEvent* aMessage = new TPolevaultAppleEvent;		// create the message.
		
		aMessage->IPolevaultAppleEvent(kAECmdSuiteID, (AEEventID)kAEQuit, itsDoc->hostAddr[whichServer], kAENoReply | kAEDontReconnect, itsDoc->GetPortRefNum() );
		fMessage = aMessage;
		fi.Success();
	}
	else
	{
		this->Free();
		fi.ReSignal();
	}
}


//--------------------------------------------------------------------------------------------------
// kAENull
//--------------------------------------------------------------------------------------------------

#pragma segment ACommand
pascal void TWakeupCommand::IWakeupCommand(TDocumentPolevault* itsDoc, int whichServer)
{
	this->IPolevaultClientCommand(itsDoc, kAENull, NULL, whichServer);	

	long clientSessionID = itsDoc->fPortRefNum;

	fMessage->WriteParameterPtr('ssID',typeLongInteger,(Ptr)&clientSessionID,sizeof(long));
}
//--------------------------------------------------------------------------------------------------
// kAEHeartBeat
//--------------------------------------------------------------------------------------------------

#pragma segment ACommand
pascal void THeartBeatCommand::IHeartBeatCommand(TDocumentPolevault* itsDoc, int whichServer)
{
	this->IClientCommand(cRemoteCommand, itsDoc, kCantUndo, kCausesChange, NULL);

	FailInfo fi;
	if (fi.Try())
	{
		TPolevaultAppleEvent* aMessage = new TPolevaultAppleEvent;		// create the message.
		
		aMessage->IPolevaultAppleEvent(kAECmdSuiteID, (AEEventID)kAEHeartBeat, itsDoc->hostAddr[whichServer], kAENoReply, itsDoc->GetPortRefNum() );

		fMessage = aMessage;
		fi.Success();
	}
	else
	{
		this->Free();
		fi.ReSignal();
	}

}

