// UTrackerMandel.cp
// Copyright 1991 Apple Computer, Inc.

#ifndef __UTRACKERMandel__
#include "UTrackerMandel.h"
#endif

#ifndef __UDOCUMENTPOLEVAULT__
#include "UDocumentPolevault.h"
#endif

/****************************** Constants ******************************/

const CommandNumber cTrackerCommand = 403;

/****************************** TTrackerMandel ******************************/

//--------------------------------------------------------------------------------------------------
#pragma segment foo

pascal void TTrackerMandel::Initialize() // override 
{
	inherited::Initialize();

	fDocument = NULL;
	fConstrainsMouse = true;		// squares only
}

//--------------------------------------------------------------------------------------------------
#pragma segment foo

pascal void TTrackerMandel::ITrackerMandel(TDocumentPolevault* itsDocument,
											TView* itsView,
											TScroller* itsScroller,
											const VPoint& itsMouse)
{
	this->ITracker(cTrackerCommand,itsDocument,kCanUndo,kDoesNotCauseChange, NULL, 
					itsView, itsScroller, itsMouse);
	fDocument = itsDocument;
}

// Override this if you want to constrain the mouse CPoint to a grid, force drawing
// a square, etc. This is called only if fConstrainsMouse is TRUE.  

#define abs(x) ((x)>0 ? (x) : -(x))
#define sign(x) ((x)>0 ? 1 : -1)
#define min(x,y) ((x)>(y)? (y):(x))
#define max(x,y) ((x)<(y)? (y):(x))

pascal void TTrackerMandel::TrackConstrain(TrackPhase /* aTrackPhase */,
										const VPoint& anchorPoint,
									  	const VPoint& /* previousPoint */,
									    VPoint& nextPoint,
										Boolean mouseDidMove)
{
	/* FIXME constrain to drawn area */
	
	if (mouseDidMove)
	{
		VPoint delta = nextPoint - anchorPoint;

		if (abs(delta.h) < abs(delta.v))
			delta.v = abs(delta.h) * sign(delta.v);
		else
			delta.h = abs(delta.v) * sign(delta.h);

		nextPoint = anchorPoint + delta;
	}
}


//--------------------------------------------------------------------------------------------------
#pragma segment ADoCommand

pascal void TTrackerMandel::DoIt() // override 
{
	VPoint topleft( min(fAnchorPoint.h, fLastNextPoint.h), min(fAnchorPoint.v, fLastNextPoint.v) );
	VPoint botright( max(fAnchorPoint.h, fLastNextPoint.h), max(fAnchorPoint.v, fLastNextPoint.v) );
	
	if (botright.h - topleft.h > 8)
		fDocument->Zoom(topleft, botright);
}

//--------------------------------------------------------------------------------------------------
#pragma segment ADoCommand

pascal void TTrackerMandel::TrackFeedback(TrackPhase aTrackPhase,
										const VPoint& anchorPoint,
										const VPoint& previousPoint,
										const VPoint& nextPoint,
										Boolean mouseDidMove,
										Boolean turnItOn) // override 
{
	inherited::TrackFeedback(aTrackPhase, anchorPoint, previousPoint, nextPoint, mouseDidMove, turnItOn); 
}


//--------------------------------------------------------------------------------------------------
#pragma segment ADoCommand

pascal TTracker* TTrackerMandel::TrackMouse(TrackPhase aTrackPhase,
											 VPoint& anchorPoint,
											 VPoint& previousPoint,
											 VPoint& nextPoint,
											 Boolean mouseDidMove) // override 
{
	return inherited::TrackMouse(aTrackPhase,anchorPoint, previousPoint, nextPoint,mouseDidMove);
}

//--------------------------------------------------------------------------------------------------
#pragma segment ADoCommand

pascal void TTrackerMandel::RedoIt() // override 
{
	inherited::RedoIt();	// TCommand::ReDoit simply calls DoIt, but this is not always adequate
}

//--------------------------------------------------------------------------------------------------
#pragma segment ADoCommand

pascal void TTrackerMandel::UndoIt() // override 
{
	SysBeep(2);
	SysBeep(2);
}

